<?php
class ModelPaymentPartpay extends Model {
	private $environments;

	public function init_environment_config() {
		if (!isset($this->environments) || empty($this->environments) ) {
			$target_environment = !$this->config->get('partpay_test') ? 'prod' : 'ci';
			$target_locale = $this->config->get('partpay_locale');
			$this->environments = $this->get_environment_config($target_environment, $target_locale);
			
			$this->log->write('PP: init_environment_config');
			$this->log->write($this->environments);
		}
 	}

	public function getMethod($address, $total) {
		$this->log->write('PP: getMethod');
		$this->load->language('payment/partpay');
		$this->init_environment_config();

		$method_data = array();

		if($total < 1000 && $total > 50) {
			$method_data = array(
				'code' => 'partpay',
				'title' => $this->environments['paymentTextTitle'],
				'terms'      => '',
				'sort_order' => $this->config->get('partpay_sort_order')
			);
		}
		return $method_data;
	}

	/**
	 * Get a token from auth0 so we can do stuff - "login"
	 */
	public function getAuthorizationCode() {
		$this->init_environment_config();

		$AuthURL = $this->environments['authUrl'];
		$Audience = $this->environments['authAudience'];

		$AuthBody = json_decode('{
				"client_id":"'.$this->config->get('partpay_username').'", 
				"client_secret":"'.$this->config->get('partpay_password').'", 
				"audience":"'.$Audience.'", 
				"grant_type":"client_credentials"
		}');
		
		$args = array(
			'method' => 'POST',
			'headers' => array('Content-Type' => 'application/json'),
			'body' =>  $AuthBody
		);

		$this->log->write('PP: Going to fetch token...');
		$response = $this->sendCurl($AuthURL, $args, true);
		$this->log->write($response);
		// TODO: Error checking
		$response = json_decode($response);
		
		// if (is_wp_error($response) ){
		// 	$this->log('PartPay API Error ' . $response->get_error_message());
		// 	$this->display_error('Could not log into PartPay. Please try again.');
		// 	return false;
		// }
		
		// TODO: Error checking
		if(isset($response->access_token)) {
			return $response->access_token;
		} else {
			return '';
		}

		// if ($response['response']['code'] == '200') {
		// 	$this->log('Saving token to cache');
		// 	// set_transient('partpay_access_token', $body->access_token, (int)$body->expires_in);
		// 	return $body->access_token;
		// }else {
		// 	return false;
		// }
	}
	
	private function apiUrl() {
		$this->init_environment_config();
		return $this->environments['apiUrl'];
	}

	private function orderUrl() {
		return $this->apiUrl() . '/order';
	}

	public function serviceName() {
		$this->log->write('PP: serviceName');
		$this->init_environment_config();
		return $this->environments['serviceName'];
	}


	public function proceedImageUrl() {
		$this->init_environment_config();
		return $this->environments['proceedImageUrl'];
	}

	public function proceedMessage() {
		$this->init_environment_config();
		return $this->environments['proceedMessage'];
	}

	private function configurationUrl() {
		return $this->apiUrl() . '/v2/configuration';
	}

	private function mapOrderItems(&$order, &$cart) {
		$items = array();

		foreach ($this->cart->getProducts() as $product) {
			$item_price = $this->currency->format($product['price'], $order['currency_code'], false, false);
			$item_total = $this->currency->format($product['total'], $order['currency_code'], false, false);

			$items[] =
				'{
					"name":"'. htmlentities( (string)substr($product['name'], 0, 26) ) .'",
					"sku":"' . (string)substr($product['product_id'], 0, 12) . '",
					"quantity":'. strval($product['quantity']) .',
					"price":'. strval($item_price) .'
				}'
			;
		}
		return $items;
	}

	public function getCheckoutUrl($order, $cart) {
		$this->log->write('PP: getCheckoutUrl');
		// $this->log->write($order);

		// Get the authorization token
		$access_token = $this->getAuthorizationCode();

		//Process here
		$items = $this->mapOrderItems($order, $cart);
		$this->log->write($items);
		
		//calculate total shipping amount
		// if( method_exists($order, 'get_shipping_total') ){
		// 	//WC 3.0
		// 	$shipping_total = $order->get_shipping_total();
		// } else {
		// 	//WC 2.6.x
		// 	$shipping_total = $order->get_total_shipping();
		// }
		$shipping_total = $this->currency->format(
			$order['total'] - $cart->getSubTotal(), 
			$order['currency_code'], 
			false, false
		);

		$this->load->model('account/order');

		$order_totals = $this->model_account_order->getOrderTotals($order['order_id']);
		
		$tax = 0;
		$shipping = 0;
		foreach($order_totals as $total) {
			if($total['code'] == 'tax') {
				$tax += $total['value'];
			}

			if($total['code'] == 'shipping') {
				$shipping += $total['value'];
			}
		}

		$OrderBodyString = '{ 
			"amount": '. $this->currency->format($order['total'], $order['currency_code'], false, false) .',
			"consumer": {
				"mobile":  "'. $order['telephone'] .'",
				"givenNames":  "'. $order['payment_firstname'] .'",
				"surname":  "'. $order['payment_lastname'] .'",
				"email":  "'. $order['email'] .'"
			},
			"billing": {
				"addressLine1":"'. $order['payment_address_1'].'",
				"addressLine2": "'. $order['payment_address_2'].'",
				"city": "'. $order['payment_city'].'",
				"suburb": "'.$order['payment_city'].'",
				"state": "'.$order['payment_zone'].'",
				"postcode": "'.$order['payment_postcode'].'"
			},
			"shipping": {
				"addressLine1": "'. $order['shipping_address_1'] .'",
				"addressLine2": " '. $order['shipping_address_2'] .'",
				"city": "'.$order['shipping_city'].'",
				"suburb":  "'. $order['shipping_city'] .'",
				"state": "'.$order['shipping_zone'].'",
				"postcode": "'. $order['shipping_postcode'] .'"
			},
			"description": "string",
			"items": ['
		;
		foreach ($items as $i=>$item) {
			$OrderBodyString .= $item . (($i < count($items)-1) ? ',' : '');
		}
		// $this->get_return_url( $order )
		$OrderBodyString .= '],	
			"merchant": {
				"redirectConfirmUrl": "' . $this->url->link('partpay/partpay/success') . '&status=confirmed",
				"redirectCancelUrl": "'  . $this->url->link('partpay/partpay/cancel') . '&status=cancelled",
				"statusCallbackUrl": "'  . $this->url->link('partpay/partpay/status') . '&status=status"
			},
			"merchantReference": "'. $order['order_id'] .'",
			"token": "'. $order['order_id'] .'",
			"taxAmount": '. $this->currency->format($tax, $order['currency_code'], false, false) .',
			"shippingAmount":'. $this->currency->format($shipping, $order['currency_code'], false, false) .'
		}';

		// $this->log->write($OrderBodyString);
		$OrderBody = json_decode($OrderBodyString);
		// $this->log->write($OrderBody);

		$order_args = array(
			'method' => 'POST',
			'headers' => array(
				'Content-Type' => 'application/json', 
				'Authorization' => 'Bearer '. $access_token
			),
			'body' => $OrderBody,
			'timeout' => 30
		);

		$this->log->write('POST Order request: ' . print_r($order_args, true));

		// $order_response = wp_remote_post($APIURL, $order_args);
		$order_response = $this->sendCurl($this->orderUrl(), $order_args, true);
		// $order_body = json_decode(wp_remote_retrieve_body($order_response));
		$order_body = json_decode($order_response);
		
		$this->log->write('POST Order response: ' . print_r($order_response, true));
		
		// if ($access_token == false) {
		// 	// Couldn't generate token
		// 	$order->add_order_note(__('Unable to generate the order token. Payment couldn\'t proceed.', 'woo_partpay'));
		// 	wc_add_notice(__('Sorry, there was a problem preparing your payment.', 'woo_partpay'),'error');
		// 	return array(
		// 			'result' => 'failure',
		// 			'redirect' => $order->get_checkout_payment_url(true)
		// 	);
		// } else {
		$this->log->write('Created PartPay OrderId: '.print_r($order_body,true) );

			// Order token successful, save it so we can confirm it later
			// update_post_meta($order_id,'_partpay_order_token',$order_body->token);
			// update_post_meta($order_id,'_partpay_order_id',$order_body->orderId);
			// update_post_meta($order_id,'_order_redirectURL', $order_body->redirectUrl);
			// $savedId = get_post_meta($order_id, '_partpay_order_id', true);

		// $this->log->write( 'Saved '.$savedId.' into post meta' );
		// }

		if( isset($order_body->redirectUrl) ) {
			return array(
				'result' 	=> 'success',
				'redirect'	=> $order_body->redirectUrl
			);
		} else {
			return array(
				'result' 	=> 'fail',
				'redirect'	=> '',
				'errors' => $order_body->Errors
			);
		}
	}

	public function sendCurl($url, $data, $is_post=true) {
		$this->log->write('Send request to ' . $url);
		$ch = curl_init($url);

		$allHeaders = array();
		foreach($data['headers'] as $key => $value) {
			$allHeaders[] = $key . ': ' . $value;
		}
		curl_setopt($ch, CURLOPT_HTTPHEADER, $allHeaders);
		$this->log->write($allHeaders);

		if ($is_post) {
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data['body']));
		} else {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
		}
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 60);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
		curl_setopt($ch, CURLOPT_FORBID_REUSE, 1);
		curl_setopt($ch, CURLOPT_FRESH_CONNECT, 1);

		// $this->log->write(json_encode($data['body']));
		$response = curl_exec($ch);

		if (curl_errno($ch) != CURLE_OK) {
			$response = new stdClass();
			$response->Errors = "POST Error: " . curl_error($ch) . " URL: $url";
			$response = json_encode($response);
		} else {
			$info = curl_getinfo($ch);
			// $this->log->write($info);
			if ($info['http_code'] != 200 && $info['http_code'] != 201) {
				$response = new stdClass();
				if ($info['http_code'] == 401 || $info['http_code'] == 404 || $info['http_code'] == 403) {
					$response->Errors = "Please check the API Key and Password";
				} else {
					$response->Errors = 'Error connecting to partpay: ' . $info['http_code'];
				}
				$response = json_encode($response);
			}
		}
		curl_close($ch);

		// $this->log->write($response);
		return $response;
	}

	function get_environment_config($target_environment, $target_locale) {
		$config_url = 'https://partpayassets.blob.core.windows.net/plugins/%s/opencart/%s/index.json';
		$config_url = sprintf($config_url, $target_locale == 'uk' ? 'uk' : 'nz', $target_environment == 'prod' ? 'prod' : 'ci');
	
		$ctx = curl_init();
		curl_setopt($ctx, CURLOPT_URL, $config_url);
		curl_setopt($ctx, CURLOPT_FAILONERROR, true);
		curl_setopt($ctx, CURLOPT_RETURNTRANSFER, true);
		
		$jsonStr = curl_exec($ctx);
		curl_close($ctx);
	
		return json_decode($jsonStr, true);
	}

}

